@@ -75,135 +75,7 @@ async def callback(message: aio_pika.IncomingMessage):
             username = message.routing_key.split(".")[4]
             server_path = container_files_path / "server"
             # command = body.decode('utf-8')
-            if command == "start":
-                if not running:
-                    # make sure to start the /Apfell/server in the background
-                    os.chmod(server_path, mode=0o777)
-                    output = ""
-                    process = subprocess.Popen(
-                        str(server_path),
-                        stdout=subprocess.PIPE,
-                        stderr=subprocess.STDOUT,
-                        cwd=str(container_files_path),
-                    )
-                    thread = _thread.start_new_thread(deal_with_stdout, ())
-                    time.sleep(3)
-                    process.poll()
-                    if process.returncode is not None:
-                        # this means something went wrong and the process is dead
-                        running = False
-                        await send_status(
-                            message="Failed to start\nOutput: {}".format(output),
-                            routing_key="c2.status.{}.stopped.start.{}".format(
-                                hostname, username
-                            ),
-                        )
-                        output = ""
-                    else:
-                        running = True
-                        await send_status(
-                            message="Started with pid: {}...\nOutput: {}".format(
-                                str(process.pid), output
-                            ),
-                            routing_key="c2.status.{}.running.start.{}".format(
-                                hostname, username
-                            ),
-                        )
-                        output = ""
-                else:
-                    await send_status(
-                        message="Already running...\nOutput: {}".format(output),
-                        routing_key="c2.status.{}.running.start.{}".format(
-                            hostname, username
-                        ),
-                    )
-                    output = ""
-            elif command == "stop":
-                if running:
-                    try:
-                        process.kill()
-                        process.communicate()
-                    except Exception as e:
-                        pass
-                    try:
-                        thread.exit()
-                    except Exception as e:
-                        pass
-                    running = False
-                    await send_status(
-                        message="Process killed...\nOld Output: {}".format(output),
-                        routing_key="c2.status.{}.stopped.stop.{}".format(
-                            hostname, username
-                        ),
-                    )
-                    output = ""
-                else:
-                    await send_status(
-                        message="Process not running...\nOld Output: {}".format(output),
-                        routing_key="c2.status.{}.stopped.stop.{}".format(
-                            hostname, username
-                        ),
-                    )
-                    output = ""
-                # make sure to stop the /Apfell/server in the background
-            elif command == "status":
-                if running:
-                    await send_status(
-                        message="Output: {}".format(output),
-                        routing_key="c2.status.{}.running.status.{}".format(
-                            hostname, username
-                        ),
-                    )
-                    output = ""
-                else:
-                    await send_status(
-                        message="C2 is not running",
-                        routing_key="c2.status.{}.stopped.status.{}".format(
-                            hostname, username
-                        ),
-                    )
-            elif command == "get_config":
-                try:
-                    path = container_files_path / "config.json"
-                    file_data = open(path, "rb").read()
-                except Exception as e:
-                    file_data = b"File not found"
-                encoded_data = json.dumps(
-                    {
-                        "filename": "config.json",
-                        "data": base64.b64encode(file_data).decode("utf-8"),
-                    }
-                )
-                await send_status(
-                    message=encoded_data,
-                    routing_key="c2.status.{}.{}.get_config.{}".format(
-                        hostname, "running" if running else "stopped", username
-                    ),
-                )
-            elif command == "writefile":
-                try:
-                    message = json.loads(message.body.decode("utf-8"))
-                    file_path = container_files_path / message["file_path"]
-                    file_path = file_path.resolve()
-                    if container_files_path not in file_path.parents:
-                        response = {
-                            "status": "error",
-                            "error": "trying to break out of path",
-                        }
-                    else:
-                        file = open(file_path, "wb")
-                        file.write(base64.b64decode(message["data"]))
-                        file.close()
-                        response = {"status": "success", "file": message["file_path"]}
-                except Exception as e:
-                    response = {"status": "error", "error": str(e)}
-                await send_status(
-                    message=json.dumps(response),
-                    routing_key="c2.status.{}.{}.writefile.{}".format(
-                        hostname, "running" if running else "stopped", username
-                    ),
-                )
-            elif command == "sync_classes":
+            if command == "sync_classes":
                 try:
                     import_all_c2_functions()
                     # c2profile = {}
@@ -232,6 +104,109 @@ async def callback(message: aio_pika.IncomingMessage):
             print("Failed overall message processing: " + str(e))
             sys.stdout.flush()
 
+async def get_status(request):
+    global running
+    global process
+    global output
+    output_message = {}
+    if process.poll() is None:
+        running = True
+    else:
+        running = False
+        try:
+            process.kill()
+        except Exception as e:
+            pass
+    output_message = {
+        "running": running,
+        "output": output,
+        "status": "success"
+    }
+    output = ""
+    return output_message
+
+
+async def start_profile(request):
+    global running
+    global process
+    global thread
+    global output
+    global container_files_path
+    server_path = container_files_path / "server"
+    output_message = {}
+    if not running:
+        os.chmod(server_path, mode=0o777)
+        output = ""
+        process = subprocess.Popen(
+            str(server_path),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+            cwd=str(container_files_path),
+        )
+        thread = _thread.start_new_thread(deal_with_stdout, ())
+        time.sleep(3)
+        process.poll()
+        if process.returncode is not None:
+            # this means something went wrong and the process is dead
+            running = False
+            output_message = {
+                "status": "success",
+                "running": running,
+                "output": "Failed to start\nOutput: {}".format(output)
+
+            }
+            output = ""
+        else:
+            running = True
+            output_message = {
+                "status": "success",
+                "running": running,
+                "output": "Started with pid: {}...\nOutput: {}".format(str(process.pid), output)
+
+            }
+            output = ""
+    else:
+        output_message = {
+            "status": "success",
+            "running": running,
+            "output": "Already running...\nOutput: {}".format(output)
+        }
+        output = ""
+    return output_message
+
+
+async def stop_profile(request):
+    global running
+    global process
+    global thread
+    global output
+    output_message = {}
+    if running:
+        try:
+            process.kill()
+            process.communicate()
+        except Exception as e:
+            pass
+        try:
+            thread.exit()
+        except Exception as e:
+            pass
+        running = False
+        output_message = {
+            "status": "success",
+            "output": "Process killed...\nOld Output: {}".format(output),
+            "running": running
+        }
+        output = ""
+    else:
+        output_message = {
+            "status": "success",
+            "running": running,
+            "output": "Process not running...\nOld Output: {}".format(output)
+        }
+        output = ""
+    return output_message
+
 
 async def sync_classes():
     global running
@@ -258,6 +233,52 @@ async def sync_classes():
         )
 
 
+async def get_file(request):
+    global container_files_path
+    global running
+    try:
+        path = container_files_path / request["filename"]
+        file_data = open(path, "rb").read()
+    except Exception as e:
+        file_data = b"File not found"
+    return{
+        "status": "success",
+        "running": running,
+        "filename": str(path),
+        "data": base64.b64encode(file_data).decode("utf-8"),
+    }
+
+
+async def write_file(request):
+    global container_files_path
+    global running
+    try:
+        file_path = container_files_path / request["file_path"]
+        file_path = file_path.resolve()
+        if container_files_path not in file_path.parents:
+            response = {
+                "status": "error",
+                "error": "trying to break out of path",
+                "running": running
+            }
+        else:
+            file = open(file_path, "wb")
+            file.write(base64.b64decode(message["data"]))
+            file.close()
+            response = {
+                "status": "success", 
+                "filename": str(file_path),
+                "running": running
+            }
+    except Exception as e:
+        response = {
+            "status": "error", 
+            "error": str(e),
+            "running": running
+        }
+    return response
+
+
 def print_flush(message: str):
     print(message)
     sys.stdout.flush()
@@ -322,6 +343,66 @@ async def connect_and_consume_rpc(main_config: dict, debug):
         print_flush("Exception in connect_and_consume_rpc .consume: {}".format(str(e)))
 
 
+async def rabbit_mythic_c2_rpc_callback(
+    exchange: aio_pika.Exchange, message: aio_pika.IncomingMessage
+):
+    with message.process():
+        try:
+            request = json.loads(message.body.decode())
+            response = await globals()[request["action"]](request)
+        except Exception as e:
+            print(str(e))
+            sys.stdout.flush()
+            response = {"status": "error", "error": str(e)}
+        try:
+            await exchange.publish(
+                aio_pika.Message(body=json.dumps(response).encode(), correlation_id=message.correlation_id),
+                routing_key=message.reply_to,
+            )
+        except Exception as e:
+            print(
+                "[-] Exception trying to send message back to container for rpc! " + str(e)
+            )
+            sys.stdout.flush()
+
+
+async def connect_and_consume_mythic_rpc(main_config: dict, debug):
+    connection = None
+    global hostname
+    while connection is None:
+        try:
+            if debug:
+                print_flush("Connecting to rabbitmq in connect_and_consume_mythic_rpc")
+            connection = await aio_pika.connect_robust(
+                host=main_config["host"],
+                login=main_config["username"],
+                password=main_config["password"],
+                virtualhost=main_config["virtual_host"]
+            )
+            if debug:
+                print_flush("Successfully connected in connect_and_consume_mythic_rpc")
+        except (ConnectionError, ConnectionRefusedError) as c:
+            print_flush("Connection to rabbitmq failed, trying again...")
+        except Exception as e:
+            print_flush("Exception in connect_and_consume_mythic_rpc connect: {}\n trying again...".format(str(e)))
+        await asyncio.sleep(5)
+    channel = await connection.channel()
+    # get a random queue that only the apfell server will use to listen on to catch all heartbeats
+    if debug:
+        print_flush("Declaring container specific mythic rpc queue in connect_and_consume_mythic_rpc")
+    queue = await channel.declare_queue("{}_mythic_rpc_queue".format(hostname), auto_delete=True)
+    await channel.set_qos(prefetch_count=10)
+    try:
+        if debug:
+            print_flush("Starting to consume callbacks in connect_and_consume_mythic_rpc")
+        task = await queue.consume(
+            partial(rabbit_mythic_c2_rpc_callback, channel.default_exchange)
+        )
+        result = await asyncio.wait_for(task, None)
+    except Exception as e:
+        print_flush("Exception in connect_and_consume_rpc .consume: {}".format(str(e)))
+
+
 async def mythic_service(debug: bool):
     global hostname
     global exchange
